@@ -18,8 +18,13 @@ smacofIndDiff <- function(delta, ndim = 2, weightmat = NULL, init = NULL, metric
   if (!is.list(diss)) diss <- list(diss)
   if ((is.matrix(diss[[1]])) || (is.data.frame(diss[[1]]))) diss <- lapply(diss, strucprep)
 
-  if (is.null(weightmat)) wgths <- initWeights(diss) else  wgths <- as.dist(weightmat)
-  if (!is.list(wgths)) wgths <- list(wgths)
+  if (is.null(weightmat)) wgths <- initWeights(diss)
+  if (!is.list(wgths)) {
+    wgths <- list(wgths)
+    if (length(wgths) != length(diss)) wgths <- sapply(diss, function(wwr) return(wgths))
+  }
+  if ((is.matrix(wgths[[1]])) || (is.data.frame(wgths[[1]]))) wgths <- lapply(wgths, strucprep)  
+  
   
   n <- attr(diss[[1]],"Size")
   if (p > (n - 1)) stop("Maximum number of dimensions is n-1!")
@@ -39,9 +44,9 @@ smacofIndDiff <- function(delta, ndim = 2, weightmat = NULL, init = NULL, metric
   dh <- list()
       
   for (j in 1:m) {                              #initialize weights, V, norm d as lists
-	wr <- appendList(wr,vmat(wgths[[j]]))              
-	vr <- appendList(vr,myGenInv(wr[[j]]))
-	dh <- appendList(dh,normDissN(diss[[j]],wgths[[j]],1))
+	  wr <- appendList(wr,vmat(wgths[[j]]))              
+	  vr <- appendList(vr,myGenInv(wr[[j]]))
+	  dh <- appendList(dh,normDissN(diss[[j]],wgths[[j]],1))
   }
   xr <-list()                                  #configurations as list
   sold <- sf1 <- sf2 <- 0                      #stress init
@@ -76,10 +81,10 @@ smacofIndDiff <- function(delta, ndim = 2, weightmat = NULL, init = NULL, metric
     er <- list()
     sunc <- 0
     for (j in 1:m) {                          #compute B, Y, 
-	br <- appendList(br,bmat(dh[[j]],wgths[[j]],dr[[j]]))
-	yr <- appendList(yr,vr[[j]]%*%br[[j]]%*%xr[[j]])
-	er <- appendList(er,dist(yr[[j]]))
-	sunc <- sunc + sum(wgths[[j]]*(dh[[j]]-er[[j]])^2)
+    	br <- appendList(br,bmat(dh[[j]],wgths[[j]],dr[[j]]))
+	    yr <- appendList(yr,vr[[j]]%*%br[[j]]%*%xr[[j]])
+	    er <- appendList(er,dist(yr[[j]]))
+	    sunc <- sunc + sum(wgths[[j]]*(dh[[j]]-er[[j]])^2)
     }
     scon<-sunc
 
@@ -96,8 +101,8 @@ smacofIndDiff <- function(delta, ndim = 2, weightmat = NULL, init = NULL, metric
 			   z<-z+wr[[j]]%*%yr[[j]]
 			   u<-u+wr[[j]]
 		    }
-		   aconf<-myGenInv(u)%*%z
-       yr<-repList(aconf,m)
+		    aconf<-myGenInv(u)%*%z
+        yr<-repList(aconf,m)
 	     }
 
        #-- configuration weights diagonal
@@ -123,15 +128,16 @@ smacofIndDiff <- function(delta, ndim = 2, weightmat = NULL, init = NULL, metric
 		    for (j in 1:m) {
 			    aux1<-crossprod(aconf,wr[[j]]%*%yr[[j]])
 			    aux2<-crossprod(aconf,wr[[j]]%*%aconf)
-			    auxb<-solve(aux2,aux1); bconf[[j]]<-auxb
+			    auxb<-solve(aux2,aux1)
+          bconf[[j]]<-auxb
 			    auxc<-crossprod(t(auxb))
 			    aux0<-aux0+(wr[[j]]%*%yr[[j]]%*%t(auxb))	
 			    auxk<-auxk+kronecker(auxc,wr[[j]])    			    
-		   }
-		   auxv<-kronecker(diag(p),matrix((1/n),n,n))
-		   aconf<-matrix(solve(auxk+auxv,as.vector(aux0)),n,p)
-		   for (j in 1:m)  yr[[j]]<-aconf%*%bconf[[j]]
-	    }
+		    }
+		    auxv<-kronecker(diag(p),matrix((1/n),n,n))
+		    aconf<-matrix(solve(auxk+auxv,as.vector(aux0)),n,p)
+		    for (j in 1:m)  yr[[j]]<-aconf%*%bconf[[j]]
+	     }
 
       for (j in 1:m) {
 		    er <- appendList(er,dist(yr[[j]]))
@@ -139,32 +145,30 @@ smacofIndDiff <- function(delta, ndim = 2, weightmat = NULL, init = NULL, metric
       }
 
     }
-    
     #-------- end constraints -----------
     
     snon <- scon                                
 
     #-------- nonmetric MDS ----------
     if (!metric) {
-	if ((itel%%modulus) == 0) {
-          snon<-0
-          dh<-list()
-	  for(j in 1:m) {
-		ds <- diss[[j]]
+	    if ((itel%%modulus) == 0) {
+        snon<-0
+        dh<-list()
+	       for(j in 1:m) {
+		            ds <- diss[[j]]
                 es <- er[[j]]
                 ws <- wgths[[j]]
-		if (ties=="primary") do <- monregP(ds,es,ws)
-		if (ties=="secondary") do <- monregS(ds,es,ws)
-		if (ties=="tertiary") do <- monregT(ds,es,ws)
-		dh <- appendList(dh,normDissN(do,ws,1))
-		snon <- snon+sum(ws*(dh[[j]]-es)^2)
-	  }
-        }
+	            	if (ties=="primary") do <- monregP(ds,es,ws)
+	            	if (ties=="secondary") do <- monregS(ds,es,ws)
+	            	if (ties=="tertiary") do <- monregT(ds,es,ws)
+		            dh <- appendList(dh,normDissN(do,ws,1))
+		            snon <- snon+sum(ws*(dh[[j]]-es)^2)
+	       }
+      }
     }
     #--------- end nonmetric MDS ----
     
-    if (verbose) cat("Iteration: ",formatC(itel,width=3, format="d")," Stress (not normalized): ",
-	formatC(c(snon),digits=8,width=12,format="f"),"\n")
+    if (verbose) cat("Iteration: ",formatC(itel,width=3, format="d")," Stress (not normalized): ", formatC(c(snon),digits=8,width=12,format="f"),"\n")
 
     if (((sold-snon)<eps) || (itel == itmax)) break()         #convergence
     
@@ -185,6 +189,8 @@ smacofIndDiff <- function(delta, ndim = 2, weightmat = NULL, init = NULL, metric
     dh[[i]] <- structure(dh[[i]], Size = n, call = quote(as.dist.default(m=b)), class = "dist", Diag = FALSE, Upper = FALSE) 
     attr(dh[[i]], "Labels") <- attr(er[[i]], "Labels") <- labels(diss[[i]])
   }
+  
+  
   colnames(aconf) <- cnames
   rownames(aconf) <- labels(diss[[1]])
   names(bconf) <- names(dh)
@@ -196,24 +202,26 @@ smacofIndDiff <- function(delta, ndim = 2, weightmat = NULL, init = NULL, metric
   
   if (metric) snon <- NULL          #no non-metric stress
   if (!metric) ssma <- NULL  
- # if (!metric) sold <- NULL
- # if (is.null(constraint)) scon <- NULL
   
   confdiss <- rep(list(NULL), m)
   for (j in 1:m) {                              #initialize weights, V, norm d as lists
 	 confdiss[[j]] <- normDissN(er[[j]], wgths[[j]], 1)
   }
 
- 
   resmat <- as.matrix(sumList(dh) - sumList(confdiss))^2 ##point stress
   spp <- colMeans(resmat)
+  
+  reslist <- mapply(function(ldh, lcd) {(as.matrix(ldh) - as.matrix(lcd))^2}, dh, confdiss, SIMPLIFY = FALSE) 
+  sps <- sapply(reslist, mean)
 
   if (itel == itmax) warning("Iteration limit reached! Increase itmax argument!")
   
   #return configurations, configuration distances, normalized observed distances 
   result <- list(delta = diss, obsdiss = dh, confdiss = confdiss, conf = yr, gspace = aconf, cweights = bconf,
-                 stress.m = ssma, stress.nm = snon, stress.uc = sunc, spp = spp, ndim = p, model = "Three-way SMACOF", niter = itel, nobj = n, metric = metric, call = match.call()) 
+                 stress.m = ssma, stress.nm = snon, stress.uc = sunc, spp = spp, sps = sps, ndim = p, model = "Three-way SMACOF", niter = itel, nobj = n, metric = metric, call = match.call()) 
   class(result) <- "smacofID"
   result 
 }
 
+
+
